<?php
/**
 * @file
 * Contains support form for miniOrange 2FA Login Module.
 */
namespace Drupal\miniorange_2fa\Form;

use Drupal\Core\Form\FormBase;
use Drupal\miniorange_2fa\MoAuthUtilities;

 /* Showing LoginSetting form info. */

class MoAuthLoginSettings extends FormBase
{    public function getFormId() {
        return 'miniorange_2fa_login_settings';
    }

    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {
        global $base_url ;

        $form['markup_top_2'] = array(
            '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">'
        );

        $disabled = False;
        if ( !MoAuthUtilities::isCustomerRegistered() ) {
            $form['header']  = array(
                '#markup' => t('<div class="mo_saml_register_message"><p>You need to <a href="'.$base_url.'/admin/config/people/miniorange_2fa/customer_setup">Register/Login</a> with miniOrange before using this module.</p></div><br>'),
            );
            $disabled = True;
        }

        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "miniorange_2fa/miniorange_2fa.admin",
                    "miniorange_2fa/miniorange_2fa.license",
                )
            ),
        );

        $form['mo_auth_enable_two_factor'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_two_factor'),
            '#title' => t('Enable Two-Factor module. <span style="color:red;">(If you disable this checkbox, Second-Factor authentication will not be invoked for any user during login.)</span>'),
            '#disabled' => $disabled,
            '#prefix' => t('<div class="mo2f-setup-header mo2f-setup-header-top-left">Enable Two-Factor module</div><div class="mo2f-info">'),
            '#suffix' => '</div><br><br>',
        );

        $form['mo_auth_redirect_user_after_login'] = array(
            '#type' => 'textfield',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_redirect_user_after_login'),
            '#attributes' => array('style' => 'width:700px', 'placeholder' => 'Enter the redirect URL'),
            '#description' => t('<strong>Note: </strong>Enter the entire URL (<em> including https:// </em>) where you want to redirect user after login.'),
            '#disabled' => $disabled,
            '#prefix' => t('<div class="mo2f-setup-header mo2f-setup-header-top-left">Redirect user after login</div>'),
            '#suffix' => '<br><br>',
        );

        $license_type = (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_type') == '')? 'DEMO' : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_type');
        $is_free = $license_type == 'DRUPAL_2FA_PLUGIN' || $license_type == 'PREMIUM' || $license_type == 'DRUPAL8_2FA_MODULE' ? FALSE : TRUE;
        $is_free_class = $is_free ? " mo2f-grayed-out" : "";

        $form['mo_auth_enforce_inline_registration'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enforce_inline_registration'),
            '#prefix' => t('<div class="mo2f-setup-header mo2f-setup-header-top-left">Invoke Inline Registration to setup 2nd factor for users '. MoAuthUtilities::mo_add_premium_tag() .'</span></div><div class="mo2f-info ' . $is_free_class . '">'),
            '#suffix' => '</div><br>',
            '#description' => t('<strong>Note:</strong> If second factor is not setup for the user trying to login, he will be asked to setup before login.'),
            '#disabled' => $is_free ,
            '#title' => t('Enforce 2 Factor registration for users at login time.'),
        );

        $form['mo_auth_enable_2fa_methods_for_inline'] = array(
          '#type' => 'checkbox',
          '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_allowed_2fa_methods'),
          '#prefix' => t('<br><div class="mo2f-setup-header"><div class="mo2f-setup-header-top-left">Allow specific 2Fa methods to configure '. MoAuthUtilities::mo_add_premium_tag() .'</div></div>'),
          '#disabled' => $is_free,
          '#title' => t('Enable allow specific 2Fa'),
          '#description' => t('<strong>Note:</strong> If you want to allow only specific 2FA methods tobe configured by users while inline 2fa setup then, enable this checkbox and select appropriate 2fa methods.</br></br>'),
        );


        $mo_get_2fa_methods = MoAuthUtilities::get_2fa_methods_for_inline_registration( FALSE );
        $selected_2fa_methods = ( array ) json_decode( \Drupal::config('miniorange_2fa.settings')->get('mo_auth_selected_2fa_methods') );

        foreach( $mo_get_2fa_methods as $sysName => $displayName ) {
          $form['mo_auth_2fa_methods_for_inline' . str_replace(' ','_', $sysName)] = array(
            '#type' => 'checkbox',
            '#title' => t( $displayName ),
            '#default_value' => array_key_exists( $sysName, $selected_2fa_methods )? TRUE : FALSE,
            '#states' => array(
              // Only show this field when the checkbox is enabled.
              'visible' => array(
                ':input[name="mo_auth_enable_2fa_methods_for_inline"]' => array('checked' => TRUE),
              ),
            ),
          );
        }

      /**
       * Enable login with Email
       */
        $form['mo_auth_two_factor_enable_login_with_email'] = array(
          '#type' => 'checkbox',
          '#disabled' => $disabled,
          '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_login_with_email'),
          '#prefix' => t('<br><div class="mo2f-setup-header"><div class="mo2f-setup-header-top-left"> Alter default login form </div></div>'),
          '#title' => t('Enable login by email address'),
          '#description' => t('<strong>Note:</strong> This option enables login by email address.'),
        );

        $form['mo_auth_two_factor_override_login_labels'] = array(
          '#type' => 'checkbox',
          '#disabled' => $disabled,
          '#title' => t('Override login form username title and description'),
          '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_override_login_labels'),
          '#description' => t('<strong>Note: </strong>This option allows you to override the login form username title/description.'),
        );

        $form['mo_auth_two_factor_username_title'] = array(
          '#type' => 'textfield',
          '#title' => t('Login form username title'),
          '#default_value' =>  \Drupal::config('miniorange_2fa.settings')->get('mo_auth_username_title'),
          '#attributes' => array('placeholder' => t('eg. Login by username/email address')),
          '#states' => array(
            'visible' => array(
              ':input[name="mo_auth_two_factor_override_login_labels"]' => array('checked' => TRUE),
            ),
          ),
          '#description' => t('<strong>Note: </strong>Override the username field title.'),
        );

        $form['mo_auth_two_factor_username_description'] = array(
          '#type' => 'textfield',
          '#title' => t('Login form username description'),
          '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_username_description'),
          '#attributes' => array('placeholder' => t('eg. You can use your username or email address to login.')),
          '#states' => array(
            'visible' => array(
              ':input[name="mo_auth_two_factor_override_login_labels"]' => array('checked' => TRUE),
            ),
          ),
          '#description' => t('<strong>Note: </strong>Override the username field description.'),
        );

        $form['markup_idp_login_note'] = array(
            '#markup' => t('<br><br><div class="mo2f-setup-header"><div class="mo2f-setup-header-top-left">Select Login Screen Options '. MoAuthUtilities::mo_add_premium_tag() .'</div></div><div class="mo2f-info ' . $is_free_class . '"><div class="mo2f-info"><div class="mo_saml_highlight_background_note"><b>Note: </b>By default 2nd Factor is enabled after password authentication.
             If you do not want to remember passwords anymore and just login with 2nd Factor, please select below option.</div>'),
        );

        $form['mo_auth_two_factor_instead_password'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_use_only_2nd_factor'),
            '#suffix' => '</div></div><br><br>',
            '#disabled' => $is_free,
            '#title' => t('Login with 2nd Factor only (No password required.)'),
            '#description' => t('<strong>Note:</strong> To use this feature make sure you have enabled the "ENFORCE 2 FACTOR REGISTRATION FOR USERS AT LOGIN TIME" feature.'),
        );

        $form['mo_auth_two_factor_invoke_2fa_depending_upon_IP'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_whitelist_IPs'),
            '#prefix' => t('<div class="mo2f-setup-header"><div class="mo2f-setup-header-top-left">Whitelist IP addresses '. MoAuthUtilities::mo_add_premium_tag() .'</div></div><div class="mo2f-info ' . $is_free_class . '"><div class="mo2f-info">'),
            '#disabled' => $is_free,
            '#title' => t('Whitelist IP addresses'),
            '#description' => t('<strong>Note:</strong> If you whitelist the IPs then 2FA will not be invoked for those IPs.'),
        );

        $form['mo_auth_two_factor_whitelist_IP'] = array(
            '#type' => 'textarea',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_whitelisted_IP_address'),
            '#disabled' => $is_free,
            '#attributes' => array(
                'placeholder' => t('Enter semicolon(;) separated IP addresses ( Format for range: lower_range - upper_range )'),
            ),
            '#states' => array(
                // Only show this field when the checkbox is enabled.
                'visible' => array(
                    ':input[name="mo_auth_two_factor_invoke_2fa_depending_upon_IP"]' => array('checked' => TRUE),
                ),
            ),
            '#suffix' => '</div></div><br>',
        );

        $form['mo_auth_two_factor_enable_role_based_2fa'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_role_based_2fa'),
            '#prefix' => t('<br><div class="mo2f-setup-header"><div class="mo2f-setup-header-top-left">Role based 2FA '. MoAuthUtilities::mo_add_premium_tag() .'</div></div><div class="mo2f-info ' . $is_free_class . '"><div class="mo2f-info">
                        <div class="mo_saml_highlight_background_note"><b>Note: </b>If you have enabled "LOGIN WITH 2ND FACTOR ONLY" feature, Second-Factor authentication will invoke for all roles.</div>'),
            '#disabled' => $is_free,
            '#title' => t('Enable role based 2FA'),
            '#description' => t("<strong>Note:</strong> If you want to enable 2FA for specific roles then, enable this checkbox and select appropriate roles.</br></br>"),
        );

        $roles_arr       = MoAuthUtilities::get_Existing_Drupal_Roles();
        $selected_roles  = ( array ) json_decode( \Drupal::config('miniorange_2fa.settings')->get('mo_auth_role_based_2fa_roles') );

        foreach( $roles_arr as $sysName => $displayName ) {
            $form['mo_auth_two_factor_role_based_2fa_roles_' . strtolower($sysName)] = array(
                '#type' => 'checkbox',
                '#default_value' => array_key_exists( $sysName, $selected_roles )? TRUE : FALSE,
                '#states' => array(
                    // Only show this field when the checkbox is enabled.
                    'visible' => array(
                        ':input[name="mo_auth_two_factor_enable_role_based_2fa"]' => array('checked' => TRUE),
                    ),
                ),
                '#title' => t($displayName),
            );
        }

        $form['Submit_LoginSettings_form'] = array(
            '#type' => 'submit',
            '#value' => t('Save Settings'),
            '#disabled' => $disabled,
            '#prefix' => '</div></div><br><br>',
            '#suffix' => '<br><br><br></div>'
        );

        MoAuthUtilities::AddsupportTab( $form, $form_state);

        return $form;
    }


    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {

        $two_factor               = $form['mo_auth_enable_two_factor']['#value'];
        $inline_registration      = $form['mo_auth_enforce_inline_registration']['#value'];
        $use_only_2nd_factor      = $form['mo_auth_two_factor_instead_password']['#value'];
        $mo_redirect_user_url     = $form['mo_auth_redirect_user_after_login']['#value'];

        $mo_enable_whitelist_IPs  = $form['mo_auth_two_factor_invoke_2fa_depending_upon_IP']['#value'];
        $mo_whitelist_IPs         = preg_replace('/\s+/', '', $form['mo_auth_two_factor_whitelist_IP']['#value'] );
        $mo_whitelist_IPs = MoAuthUtilities::check_for_valid_IPs( $mo_whitelist_IPs );

       /**
        * Enable login with email
        */
        $mo_login_with_email = $form['mo_auth_two_factor_enable_login_with_email']['#value'];
        $mo_override_login_labels = $form['mo_auth_two_factor_override_login_labels']['#value'];
        $mo_username_title = $form['mo_auth_two_factor_username_title']['#value'];
        $mo_username_description = $form['mo_auth_two_factor_username_description']['#value'];

        $mo_login_with_email_value = $mo_login_with_email == 1 ? TRUE : FALSE;
        $mo_override_login_labels_value = $mo_override_login_labels == 1 && !empty( $mo_username_title ) && !empty( $mo_username_description ) ? TRUE : FALSE;


        /**
         * Role based 2FA
         */
        $mo_enable_role_based_2fa = $form['mo_auth_two_factor_enable_role_based_2fa']['#value'];
        $roles_arr = MoAuthUtilities::get_Existing_Drupal_Roles();
        $mo_role_based_2fa_roles = array();
        foreach( $roles_arr as $sysName => $displayName ) {
            if ( $form['mo_auth_two_factor_role_based_2fa_roles_' . strtolower($sysName)]['#value'] == 1) {
                $mo_role_based_2fa_roles[$sysName] = $form['mo_auth_two_factor_role_based_2fa_roles_' . strtolower($sysName)]['#value'];
            }
        }
        $mo_role_based_2fa_roles_value  = !empty( $mo_role_based_2fa_roles ) ? json_encode( $mo_role_based_2fa_roles ) : '';

        /**
         * Allowed 2FA methods in inline registration flow
         */
        $mo_enable_2fa_methods_for_inline = $form['mo_auth_enable_2fa_methods_for_inline']['#value'];
        $mo_get_2fa_methods = MoAuthUtilities::get_2fa_methods_for_inline_registration( FALSE );

        $mo_allowed_2fa_methods = array();
        foreach( $mo_get_2fa_methods as $sysName => $displayName ) {
           if ( $form['mo_auth_2fa_methods_for_inline' . str_replace(' ','_', $sysName) ]['#value'] == 1) {
              $mo_allowed_2fa_methods[$sysName] = $displayName;
           }
        }

        $mo_allowed_2fa_methods_value   = !empty( $mo_allowed_2fa_methods ) ? json_encode( $mo_allowed_2fa_methods ) : '';

        $enforce_only_2nd_factor        = $use_only_2nd_factor == 1 ? TRUE : FALSE;
        $enable2Factor                  = $two_factor == 1 ? TRUE : FALSE;
        $enforceInlineRegistration      = ( $inline_registration == 1 || $enforce_only_2nd_factor ) ? TRUE : FALSE;
        $mo_enable_whitelist_IPs_value  = $mo_enable_whitelist_IPs == 1 && ( !empty( is_null( $mo_whitelist_IPs ) || !empty( $mo_whitelist_IPs ) ) ) ? TRUE : FALSE;
        $mo_enable_role_based_2fa_value = $mo_enable_role_based_2fa == 1 && !empty( $mo_role_based_2fa_roles ) ? TRUE : FALSE;
        $mo_enable_2fa_methods_for_inline_value = $mo_enable_2fa_methods_for_inline == 1 && !empty( $mo_allowed_2fa_methods_value ) ? TRUE : FALSE;

        $configFactory = \Drupal::configFactory()->getEditable('miniorange_2fa.settings');

        $configFactory->set('mo_auth_use_only_2nd_factor', $enforce_only_2nd_factor)->save();
        $configFactory->set('mo_auth_redirect_user_after_login', $mo_redirect_user_url)->save();
        $configFactory->set('mo_auth_enable_two_factor', $enable2Factor)->save();
        $configFactory->set('mo_auth_enforce_inline_registration', $enforceInlineRegistration)->save();

        $configFactory->set('mo_auth_enable_login_with_email', $mo_login_with_email_value)->save();
        $configFactory->set('mo_auth_override_login_labels', $mo_override_login_labels_value)->save();
        $configFactory->set('mo_auth_username_title', $mo_username_title)->save();
        $configFactory->set('mo_auth_username_description', $mo_username_description)->save();

        $configFactory->set('mo_auth_enable_whitelist_IPs', $mo_enable_whitelist_IPs_value)->save();
        $configFactory->set('mo_auth_whitelisted_IP_address', $mo_whitelist_IPs)->save();

        $configFactory->set('mo_auth_enable_role_based_2fa', $mo_enable_role_based_2fa_value)->save();
        $configFactory->set('mo_auth_role_based_2fa_roles', $mo_role_based_2fa_roles_value)->save();

        $configFactory->set('mo_auth_enable_allowed_2fa_methods', $mo_enable_2fa_methods_for_inline_value)->save();
        $configFactory->set('mo_auth_selected_2fa_methods', $mo_allowed_2fa_methods_value)->save();

        drupal_flush_all_caches();
        \Drupal::messenger()->addMessage(t("Login settings updated."), 'status');
    }
}